#!/bin/bash
##
## Sub functions to detect essentials about our environment.
## These are intended to be included by other scripts.
##
## You should not need to edit this file.
##
## Copyright (c) 2011 Dell Inc.
##



#
# Prevent needless reentry, unless desired.
#
if [ "$1" == "--redetect" ] ; then
	FN_DETECT_ENV=""	# clear it, so we redetect.
fi

if [ "$FN_DETECT_ENV" == "done" ] ; then
	TMP=1	# NOP.  Stopped re-entry
else

FN_DETECT_ENV="done"

#
# Constants / Program references
#

if [ "$MEGACLI" == "" ] ; then
	export MEGACLI="MegaCli"
fi

if [ "$IPMITOOL" == "" ] ; then
	export IPMITOOL="ipmitool"
fi

if [ "$DMIDECODE" == "" ] ; then
	export DMIDECODE="dmidecode"
fi

if [ "$BMCTOOL" == "" ] ; then
	export BMCTOOL="/opt/pec_tools/scripts/bmc"
fi

detect_bios_version() {
	RC=`$DMIDECODE | grep BIOS -A 2 | grep Version | cut -d " " -f 2`
	EC=$?
}

detect_bmc_version() {
	RC=`ipmitool mc info | grep "Firmware Revision" | cut -d":" -f2 | cut -d" " -f2`
	EC=$?
}


#
# Check that environment is sane.
#
# Check that we are root
TMP=`whoami`
if [ ! "$TMP" == "root" ] ; then
	printf "\n"
	printf "You must be root to use this.\n\n"
	exit 1
fi

# Check all utilities are in path
check_tool_exists() {
	TMP=`which $1 &> /dev/null`
	if [ $? -ne 0 ] ; then
		printf "\n"
		printf "$1 is not in path; please fix this and rerun.\n\n"
		exit 1
	fi
}

check_tool_exists $MEGACLI
check_tool_exists $IPMITOOL
check_tool_exists $DMIDECODE
check_tool_exists $BMCTOOL

#
# Warn user it may be a few seconds
#
#printf "Detecting... please wait a few seconds...\n\n"

#
# Detect platform, if it has not been set.
#
if [ "$PLATFORM" == "UNDEFINED" ] ; then
	PLATFORM=""
fi

if [ -z $PLATFORM ] ; then

	TMP=`$DMIDECODE | grep -m1 "Product Name" | cut -d":" -f2 | cut -d" " -f2- `
	case $TMP in
		*C1100*) PLATFORM="C1100" ;;
		*C2100*) PLATFORM="C2100" ;;
		*C6100*) PLATFORM="C6100" ;;
		*C6105*) PLATFORM="C6105" ;;
		*C6145*) PLATFORM="C6145" ;;
		*) PLATFORM="unknown" ;;
	esac

	export PLATFORM

	# This is a bit hackish, but it's the best way to go to cause least surprise to the
	# user.
	echo "" >> /tmp/env
	echo "# This is detected by fn_detect_env.sh, and written here." >> /tmp/env
	echo "export PLATFORM=\"$PLATFORM\"" >> /tmp/env
	echo "" >> /tmp/env
fi

#
# Get Service tag (AKA "Product Serial") and Asset tag.  Note, in shared infrastructure systems,
# this should have a suffix of .01 | .02 | .03 ... to indicate which node we're talking to 
#
SVC_TAG=`$IPMITOOL fru | grep "Product Serial" | cut -d":" -f2 | cut -d" " -f2`
ASSET_TAG=`$IPMITOOL fru | grep "Product Asset Tag" | cut -d":" -f2 | cut -d" " -f2`
# Strip unprintable chars:  These must be sane / usable.  Customer boards should never have
# issues with this, but mine do :-(
SVC_TAG=`echo $SVC_TAG | perl -i -pe 's/(.)/(ord($1)>127)?"":$1/egs;'`   
ASSET_TAG=`echo $ASSET_TAG | perl -i -pe 's/(.)/(ord($1)>127)?"":$1/egs;'` 
if [ "${#SVC_TAG}" -eq "0" ] ; then
	SVC_TAG="no_svc_tag"
fi
if [ "${#ASSET_TAG}" -eq "0" ] ; then
	ASSET_TAG="no_asset_tag"
fi

#
# Get BIOS and BMC versions.
#
###BIOSVER=`$DMIDECODE | grep BIOS -A 2 | grep Version | cut -d " " -f 2`
###BMCVER=`$IPMITOOL mc info | grep "Firmware Revi" | cut -d " " -f 12`
detect_bios_version
BIOSVER=$RC
detect_bmc_version
BMCVER=$RC


#
# Get System time, as well as BMC (SEL) time
#
SYSTIME=`date +"%m/%d/%Y %H:%M:%S"`
SYSTIMEYMD=`date +"%Y-%m-%d %H:%M:%S"`
SELTIME=`ipmitool sel time get`


#
# Get FCB info, if present
#
HAS_FCB="no"  # default
FCBVER="n/a"
FCBPIC="n/a"
case $PLATFORM in
	C6100) 
		HAS_FCB="yes"
		FCBVER=`$IPMITOOL raw 0x34 0x16 | cut -d" " -f2- `
		TMP=`$IPMITOOL raw 0x34 0x70 | cut -d" " -f2 `
		case $TMP in 
			10) FCBPIC="PIC16" ;;
			12) FCBPIC="PIC18" ;;
			*)  FCBPIC="unknown" ;;
		esac
		;;

	C6105)
		HAS_FCB="yes"
		FCBVER=`$IPMITOOL raw 0x34 0x75 | cut -d" " -f2- `
		# mjs >>> need method of detecting pic
		FCBPIC="NEED DETECT METHOD"
		;;

	C6145)
		HAS_FCB="yes"
		# mjs >>> need method of detecting pic
		FCBPIC="NEED DETECT METHOD"
		;;
esac

#
# Shared infrastructure?  Get Node ID
#
HAS_NODES="no"  #default
case $PLATFORM in
	C6100) HAS_NODES="yes" ;;	
	C6105) HAS_NODES="yes" ;;
	C6145) HAS_NODES="yes" ;;
esac

if [ "$HAS_NODES" == "yes" ] ; then
	NODEID=`$BMCTOOL node_number`
fi

#
# Get count, number of storage adapters
#
# mjs >>> currently using a lot of assumptions; one adapter; one enclosure.  Generalize/improve this.
LSI_ADPCOUNT=`$MEGACLI -AdpCount | grep "Controller Count" | cut -d" " -f3 | cut -d"." -f1`
LSI_ADPNUM=`$MEGACLI -AdpAllInfo -aALL | grep -m1 "Adapter #" | cut -d"#" -f2`
LSI_ADPNAME=`$MEGACLI -AdpAllInfo -a$LSI_ADPNUM | grep -i "Product Name" | cut -d":" -f2 | cut -d" " -f2- `
LSI_ADPFWVER=`$MEGACLI -AdpAllInfo -a$LSI_ADPNUM | grep "FW Package Build" | cut -d " " -f 4`
LSI_ENC_COUNT=`$MEGACLI -EncInfo -a0 | grep "Number of enclosures on adapter" | cut -d"-" -f3 | cut -d" " -f2`
LSI_ENC_ID=`$MEGACLI -PDList -a$LSI_ADPNUM | grep -m1 "Enclosure Device ID" | cut -d":" -f2 | cut -d" " -f2`
LSI_SLOTS=( `$MEGACLI -PDList -a$LSI_ADPNUM | grep "Slot Number" | cut -d":" -f2 | cut -d" " -f2` )
LSI_PDCOUNT="${#LSI_SLOTS[@]}"
LSI_SLOTS_SZ=()			# Drive size
LSI_SLOTS_INQDATA=()	# Drive inquiry data.  Often badly formed.
for ((i=0 ; i < $LSI_PDCOUNT ; i++))
do
	TMP1=`$MEGACLI -PDlist -a$LSI_ADPNUM | grep -B1 -A10 "Slot Number: ${i}" | grep "Raw Size" | cut -d" " -f3-4`
	TMP2=`$MEGACLI -pdlist -a$LSI_ADPNUM | grep -B1 -A20 "Slot Number: ${i}" | grep "Inquiry Data" | cut -d":" -f2 | sed 's/ *//'`
	
	LSI_SLOTS_SZ[i]=${TMP1}
	LSI_SLOTS_INQDATA[i]=${TMP2}
done

INTEG_ADPCOUNT="0"
INTEG_ADAPTERS=()
while read TMP ; do
	INTEG_ADAPTERS[INTEG_ADPCOUNT]=${TMP}
	let "INTEG_ADPCOUNT+=1"
done << EOF
`lspci | grep -i sata | grep -i controller`
EOF

#
# BMC: read users
#
BMCUSERS_COUNT="10"
BMCUSERS_NAME=()
BMCUSERS_CALLIN=()
BMCUSERS_LINKAUTH=()
BMCUSERS_IPMIMSG=()
BMCUSERS_CHANNELPRIVLIMIT=()
for ((i=1 ; i <= $BMCUSERS_COUNT ; i++)) 
do
	TMP=`$BMCTOOL users get $i`
	BMCUSERS_NAME[$i]=`echo $TMP | cut -d" " -f1`
	BMCUSERS_CALLIN[$i]=`echo $TMP | cut -d" " -f2`
	BMCUSERS_LINKAUTH[$i]=`echo $TMP | cut -d" " -f3`
	BMCUSERS_IPMIMSG[$i]=`echo $TMP | cut -d" " -f4`
	BMCUSERS_CHANNELPRIVLIMIT[$i]=`echo $TMP | cut -d" " -f5`
done

#
# BMC: in shared or dedicated ethernet mode?
#
BMC_NIC_MODE="unknown"  # default
TMP=`$BMCTOOL nic_mode get`

if [ $? == "0" ] ; then
	BMC_NIC_MODE=$TMP
fi

#
# Get BMC LAN parameters
#
BMC_IP_SRC=`$IPMITOOL lan print 1 | grep "IP Address Source" | cut -d":" -f2 | cut -d" " -f2`
BMC_IP=`$IPMITOOL lan print 1 | grep "IP Address" | grep -v "Source" | cut -d":" -f2 | cut -d" " -f2`
BMC_SUBNET_MASK=`$IPMITOOL lan print 1 | grep "Subnet Mask" | cut -d":" -f2 | cut -d" " -f2`
BMC_MAC=`$IPMITOOL lan print 1 | grep "MAC Address" | cut -d":" -f2- | cut -d" " -f2`
BMC_DEFAULT_GW_IP=`$IPMITOOL lan print 1 | grep "Default Gateway IP" | cut -d":" -f2 | cut -d" " -f2`
BMC_DEFAULT_GW_MAC=`$IPMITOOL lan print 1 | grep "Default Gateway MAC" | cut -d":" -f2- | cut -d" " -f2`
BMC_VLAN_ID=`$IPMITOOL lan print 1 | grep "802.1q VLAN ID" | cut -d":" -f2 | cut -d" " -f2`
BMC_VLAN_PRIORITY=`$IPMITOOL lan print 1 | grep "802.1q VLAN Priority" | cut -d":" -f2 | cut -d" " -f2`


#
# Get information on all connected ethernet adapters
#
ETH_ADPCOUNT=`ifconfig -a | grep ^eth | wc -l`
ETH_MACS=()
for ((i=0 ; i < $ETH_ADPCOUNT ; i++))
do
	TMP=`ifconfig eth${i} | grep HWaddr | awk '{ print $5; }'`
	# Swap out periods for colons, so we dont confuse Windows
	ETH_MACS[i]=${TMP//\:/.}
done

ETH0_IP=`ifconfig eth0 | grep "inet addr" | cut -d":" -f2 | cut -d " " -f 1 | sed -e 's/ *$//' | sed -e 's/^ *//'`
ETH1_IP=`ifconfig eth1 | grep "inet addr" | cut -d":" -f2 | cut -d " " -f 1 | sed -e 's/ *$//' | sed -e 's/^ *//'`

##
## Titanium-specific
##

# Check for HIC board
HAS_HIC="no"
TMP=`lspci | grep "Tesla S870" | wc -l`
if [ "$TMP" -gt "0" ] ; then
	HAS_HIC="yes"
fi

# Check for ipass board
HAS_IPASS="no"
TMP=`lspci | grep "PLX" | grep "Unknown device 8647" | wc -l`
if [ "$TMP" -gt "0" ] ; then
	HAS_IPASS="yes"
fi

# Check for midplane
HAS_MIDPLANE="no"
TMP=`lspci | grep "PLX" | grep "Unknown device 8696" | wc -l`
if [ "$TMP" -gt "0" ] ; then
	HAS_MIDPLANE="yes"
fi

# Get the number of GPGPUs seen on the PCI bus.
GPU_COUNT=`lspci | grep "3D controller" | wc -l`


#
# Construct a unique identifier for this system. Service tag + LOM0 MAC works well
#
SYSTEMID="${SVC_TAG}_${ETH_MACS[0]}"

#
# Utility function to get the date/time
#
now() {
	RC=`date +"%Y-%m-%d %H:%M:%S"`
}


fi  # (stop reentry)
